@@ -1,6 +1,7 @@
 #![cfg_attr(feature = "nightly", feature(array_try_map))]
 #![doc = include_str!("../README.md")]
 
+mod bytes;
 mod cursor;
 pub mod lencoder;
 mod record;
@@ -8,11 +9,13 @@ mod types;
 
 use core::convert::TryInto;
 use core::mem::{size_of, MaybeUninit};
-use core::ptr;
+use core::{fmt, ptr};
 use ErrorKind::*;
 
+pub use bytes::*;
 pub use cursor::*;
 pub use derive::*;
+pub use lencoder::Lencoder;
 pub use record::*;
 
 /// Shortcut for `Result<T, bin_layout::ErrorKind>`
@@ -32,20 +35,39 @@ pub enum ErrorKind {
     Other,
 }
 
+impl fmt::Display for ErrorKind {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Debug::fmt(&self, f)
+    }
+}
+impl std::error::Error for ErrorKind {}
+
 /// A trait for serialize and deserialize data for binary format.
 ///
 /// All [primitive types](https://doc.rust-lang.org/stable/rust-by-example/primitives.html) implement this trait.
 ///
 /// `Vec`, `String`, `&[T]`, `&str` etc.. are encoded with their length value first, Following by each entry.
 pub trait DataType<'de>: Sized {
+    /// The size of the data type in bytes. (padding not included)
+    const SIZE: usize = size_of::<Self>();
+
+    /// If `true`, the data type could be fixed or dynamic sized,
+    ///
+    /// But if `false`, then the data type is always fixed sized.
+    const IS_DYNAMIC: bool = true;
+
+    /// Calculate total estimated size of the data structure in bytes.
+    #[inline]
+    fn size_hint(&self) -> usize {
+        Self::SIZE
+    }
+
     /// Serialize the data to binary format.
-    fn serialize(self, _: &mut Cursor<impl AsMut<[u8]>>) -> Result<()>;
+    fn serialize(self, _: &mut Cursor<impl Bytes>);
 
     /// Deserialize the data from binary format.
     fn deserialize(_: &mut Cursor<&'de [u8]>) -> Result<Self>;
 
-    /// Shortcut for `DataType::serialize(self, &mut View::new(bytes.as_mut()))`
-    ///
     /// ### Example
     ///
     /// ```
@@ -56,14 +78,14 @@ pub trait DataType<'de>: Sized {
     ///     foo: u8,
     ///     bar: [u8; 2],
     /// }
-    ///
-    /// let mut bytes = [0; 3];
-    /// FooBar { foo: 1, bar: [2, 3] }.encode(&mut bytes);
-    /// assert_eq!(bytes, [1, 2, 3]);
+    /// let foobar = FooBar { foo: 1, bar: [2, 3] }.encode();
+    /// assert_eq!(vec![1, 2, 3], foobar);
     /// ```
     #[inline]
-    fn encode(self, data: &mut [u8]) -> Result<()> {
-        self.serialize(&mut Cursor::from(data))
+    fn encode(self) -> Vec<u8> {
+        let mut vec = Vec::with_capacity(self.size_hint());
+        self.serialize(&mut (&mut vec).into());
+        vec
     }
 
     /// Shortcut for `DataType::deserialize(&mut View::new(bytes.as_ref()))`
@@ -87,14 +109,3 @@ pub trait DataType<'de>: Sized {
         Self::deserialize(&mut Cursor::from(data))
     }
 }
-
-// -----------------------------------------------------------------------
-
-macro_rules! ret_err_or_add {
-    [($offset:expr; + $count:expr) > $len:expr] => {
-        let total_len = $offset + $count;
-        if total_len > $len { return Err(InsufficientBytes); }
-        $offset = total_len;
-    };
-}
-pub(crate) use ret_err_or_add;
