@@ -3,17 +3,14 @@ use bin_layout::{lencoder::*, DataType};
 #[test]
 fn test_lencoder_l2() {
     for num in 0..=L2::MAX {
-        let mut buf = [0; 2];
-        L2(num).encode(&mut buf).unwrap();
-        assert_eq!(num, L2::decode(&buf).unwrap().0);
+        // let mut buf = [0; 2];
+        assert_eq!(num, L2::decode(&L2(num).encode()).unwrap().0);
     }
 }
 #[test]
 fn test_lencoder_l3() {
-    for num in 0..(L3::MAX / 3) {
-        let mut buf = [0; 3];
-        L3(num).encode(&mut buf).unwrap();
-        assert_eq!(num, L3::decode(&buf).unwrap().0);
+    for num in 0..=(L3::MAX / 3) {
+        assert_eq!(num, L3::decode(&L3(num).encode()).unwrap().0);
     }
 }
 
